/**
 * Integration tests
 *
 * This tests things like shareable configs to ensure they are exported correctly and can be used by the end-user
 * properly.
 *
 * Note that specifying the tangerine plugin works out of the box because we're using yarn workspaces which provides a symlinked
 * version of each workspace in the root. This allows eslint to resolve the plugin using standard node module resolution.
 */
const { CLIEngine } = require('eslint');

const shouldReportError = (report, messageArgs) => {
  expect(report.results).toHaveLength(1);
  expect(report.results[0]).toMatchObject({
    errorCount: 1,
    messages: expect.arrayContaining([expect.objectContaining(messageArgs)]),
  });
};

describe('Integration tests', () => {
  const commonConfig = {
    parser: 'babel-eslint',
    plugins: ['tangerine'],
    useEslintrc: false,
  };
  describe('Configs', () => {
    it('non-existent config should error', () => {
      expect(() => {
        // eslint-disable-next-line no-unused-vars
        const cli = new CLIEngine({
          baseConfig: {
            extends: ['plugin:tangerine/doesnotexist'],
          },
          ...commonConfig,
        });
      }).toThrow(/^Failed to load config "plugin:tangerine\/doesnotexist"/);
    });
    describe('Packages config', () => {
      let cli;
      let cliWithSettings;
      beforeEach(() => {
        cli = new CLIEngine({
          baseConfig: {
            extends: ['plugin:tangerine/packages'],
          },
          ...commonConfig,
        });
        cliWithSettings = new CLIEngine({
          baseConfig: {
            extends: ['plugin:tangerine/packages'],
            settings: {
              tangerine: {
                guidelinesSitePrefix: 'https://frontend.atlassian.com',
                message: 'may the force be with you',
              },
            },
          },
          ...commonConfig,
        });
      });

      it('should not allow parent imports in src/ui directories', () => {
        const report = cli.executeOnText(
          'import { Bar } from "../bar"',
          `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/ui/foo/index.js`,
        );
        shouldReportError(report, {
          ruleId: 'tangerine/import/no-parent-imports',
          message:
            "Importing from a parent inside 'src/ui' is not allowed; For more details see https://github.com/atlassian/frontend/tree/master/guides/architecture/general/components/components-types/ui/",
          severity: 2,
        });

        const reportWithSettings = cliWithSettings.executeOnText(
          'import { Bar } from "../bar"',
          `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/ui/foo/index.js`,
        );
        shouldReportError(reportWithSettings, {
          ruleId: 'tangerine/import/no-parent-imports',
          message:
            "Importing from a parent inside 'src/ui' is not allowed; For more details see https://frontend.atlassian.com/architecture/general/components/components-types/ui/; may the force be with you",
          severity: 2,
        });
      });

      it('should not allow imports from ui inside services/controllers/common', () => {
        ['services', 'controllers', 'common'].forEach(dir => {
          const report = cli.executeOnText(
            'import { Bar } from "../../ui"',
            `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/${dir}/foo/index.js`,
          );
          shouldReportError(report, {
            ruleId: 'tangerine/import/no-restricted-paths',
            message: `Importing 'src/ui' within 'src/${dir}' is restricted; For more details see https://github.com/atlassian/frontend/tree/master/guides/architecture/general/apps/`,
            severity: 2,
          });

          const reportWithSettings = cliWithSettings.executeOnText(
            'import { Bar } from "../../ui"',
            `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/${dir}/foo/index.js`,
          );
          shouldReportError(reportWithSettings, {
            ruleId: 'tangerine/import/no-restricted-paths',
            message: `Importing 'src/ui' within 'src/${dir}' is restricted; For more details see https://frontend.atlassian.com/architecture/general/apps/; may the force be with you`,
            severity: 2,
          });
        });
      });

      it('should not allow imports between services/controllers/common', () => {
        [
          { target: 'services', from: 'controllers' },
          { target: 'controllers', from: 'services' },
          { target: 'common', from: 'services' },
          { target: 'common', from: 'controllers' },
        ].forEach(({ target, from }) => {
          const report = cli.executeOnText(
            `import { Bar } from "../../${from}"`,
            `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/${target}/foo/index.js`,
          );
          shouldReportError(report, {
            ruleId: 'tangerine/import/no-restricted-paths',
            message: `Importing 'src/${from}' within 'src/${target}' is restricted; For more details see https://github.com/atlassian/frontend/tree/master/guides/architecture/general/apps/`,
            severity: 2,
          });
        });
      });

      it('should not allow nested imports in ui/services/controllers/common', () => {
        ['services', 'controllers', 'common', 'ui'].forEach(dir => {
          const report = cli.executeOnText(
            `import { Bar } from "./foo/bar/baz.js"`,
            `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/${dir}/foo/index.js`,
          );
          shouldReportError(report, {
            ruleId: 'tangerine/import/no-nested-imports',
            message: `Importing from a descendant directory more than one level deep is not allowed inside 'src/${dir}'; For more details see https://github.com/atlassian/frontend/tree/master/guides/architecture/general/apps/`,
            severity: 2,
          });

          const reportWithSettings = cliWithSettings.executeOnText(
            `import { Bar } from "./foo/bar/baz.js"`,
            `${__dirname}/__fixtures__/multi-pkg-project/pkg-a/src/${dir}/foo/index.js`,
          );
          shouldReportError(reportWithSettings, {
            ruleId: 'tangerine/import/no-nested-imports',
            message: `Importing from a descendant directory more than one level deep is not allowed inside 'src/${dir}'; For more details see https://frontend.atlassian.com/architecture/general/apps/; may the force be with you`,
            severity: 2,
          });
        });
      });
    });
  });

  describe('Recommended config', () => {
    it('should contain rules from packages config', () => {
      const cliWithRecommended = new CLIEngine({
        baseConfig: {
          extends: ['plugin:tangerine/recommended'],
        },
        ...commonConfig,
      });

      const cliWithPackages = new CLIEngine({
        baseConfig: {
          extends: ['plugin:tangerine/packages'],
        },
        ...commonConfig,
      });
      expect(cliWithRecommended.getConfigForFile('foo.js').rules).toMatchObject(
        cliWithPackages.getConfigForFile('foo.js').rules,
      );
    });
  });
});
