const outdent = require('outdent');

const config = {
  'tangerine/i18n/messages-are-spelled-correctly': [
    'error',
    {
      language: 'en-US',
      messageSelectors: ['Literal'],
      additionalWords: {
        caseSensitive: ['Jira'],
        caseInsensitive: ['kanban'],
      },
    },
  ],
};

describe('i18n/messages-are-spelled-correctly', () => {
  const testCase = (description, { textToSpellcheck, expectedErrors }) => {
    test(description, () => {
      expect({
        config,
        code: outdent`
          /* eslint-disable import/prefer-default-export, prettier/prettier */
          export const message = "${textToSpellcheck}";
        `,
      }).toMatchEslintResults({
        messages: expectedErrors.map(message => ({ message })),
      });
    });
  };

  describe('No spelling errors', () => {
    testCase('Regular words', {
      textToSpellcheck: 'These are some regular boring words that are spelled correctly.',
      expectedErrors: [],
    });

    testCase('Symbols', {
      textToSpellcheck: 'Symbols are not spelling errors: !@#$%^&*()',
      expectedErrors: [],
    });

    testCase('Numbers', {
      textToSpellcheck: 'Numbers are fine: 1234567890',
      expectedErrors: [],
    });

    testCase('Absolute URLs', {
      textToSpellcheck: 'Here are some: http://google.com and https://example.com/foo?a=b&c=d',
      expectedErrors: [],
    });

    testCase('Fancy apostrophes', {
      textToSpellcheck: "Fancy apostrophes are fine. For example: can’t vs can't.",
      expectedErrors: [],
    });

    testCase('Variable substitution (react-intl syntax)', {
      textToSpellcheck: 'Here are some variables: {fooBarBaz} {foo_bar_baz}',
      expectedErrors: [],
    });

    testCase("Case-insensitive additional words that aren't in the dictionary", {
      textToSpellcheck: "The standard dictionary doesn't know what kanban or Kanban means!",
      expectedErrors: [],
    });

    testCase("Case-sensitive additional words that aren't in the dictionary", {
      textToSpellcheck: "The standard dictionary doesn't know about Jira either :O",
      expectedErrors: [],
    });
  });

  describe('Spelling errors', () => {
    testCase('Errors that have suggested fixes', {
      textToSpellcheck: 'Uh oh, speling is triky!',
      expectedErrors: [
        outdent`
          It looks like "speling" is spelled incorrectly. Did you mean:
          * Spelling?
          * Spewing?
          * Spieling?
          If this is a false-positive, please add "speling" to the list of additional words specified in the lint rule's configuration.
        `,
        outdent`
          It looks like "triky" is spelled incorrectly. Did you mean:
          * Trike?
          * Tricky?
          * Trixy?
          If this is a false-positive, please add "triky" to the list of additional words specified in the lint rule's configuration.
        `,
      ],
    });

    testCase('Errors that have no suggested fixes', {
      textToSpellcheck: 'Uh oh, spexfkkdling is really trcdsmliky!',
      expectedErrors: [
        'It looks like "spexfkkdling" is spelled incorrectly. If this is a false-positive, please add "spexfkkdling" to the list of additional words specified in the lint rule\'s configuration.',
        'It looks like "trcdsmliky" is spelled incorrectly. If this is a false-positive, please add "trcdsmliky" to the list of additional words specified in the lint rule\'s configuration.',
      ],
    });

    testCase('Case-sensitivity of additional words', {
      textToSpellcheck: "Uh oh, JIRA isn't capitalized correctly.",
      expectedErrors: [
        outdent`
          It looks like "JIRA" is spelled incorrectly. Did you mean:
          * Jinx?
          * Jinn?
          * Jive?
          If this is a false-positive, please add "JIRA" to the list of additional words specified in the lint rule's configuration.
        `,
      ],
    });

    testCase('Suggestions are capitalized correctly', {
      textToSpellcheck: 'HTTPP',
      expectedErrors: [
        outdent`
          It looks like "HTTPP" is spelled incorrectly. Did you mean:
          * HTTP?
          If this is a false-positive, please add "HTTPP" to the list of additional words specified in the lint rule's configuration.
        `,
      ],
    });

    testCase('Aussie 🦘 vs. American 🦅 spelling', {
      textToSpellcheck:
        "Gosh darn, I just realised we can't use Aussie spelling. What is this organisation coming to? So demoralising! I'm going to go and drown my sorrows in yoghurt.",
      expectedErrors: [
        outdent`
          It looks like "realised" is spelled incorrectly. Did you mean:
          * Realized?
          * Realism?
          * Realisms?
          If this is a false-positive, please add "realised" to the list of additional words specified in the lint rule's configuration.
        `,
        outdent`
          It looks like "organisation" is spelled incorrectly. Did you mean:
          * Organization?
          * Organizations?
          If this is a false-positive, please add "organisation" to the list of additional words specified in the lint rule's configuration.
        `,
        outdent`
          It looks like "demoralising" is spelled incorrectly. Did you mean:
          * Demoralizing?
          If this is a false-positive, please add "demoralising" to the list of additional words specified in the lint rule's configuration.
        `,
        outdent`
          It looks like "yoghurt" is spelled incorrectly. Did you mean:
          * Yogurt?
          * Yoghurt's?
          * Yogurts?
          If this is a false-positive, please add "yoghurt" to the list of additional words specified in the lint rule's configuration.
        `,
      ],
    });
  });
});
